#include <LiquidCrystal_I2C.h>
#include <Servo.h>

// Servo variables
Servo myservo;
const int servo_pin = 5;

int servoAngle = 0; // servo position
int increment = 0; // servo increment in degrees (1 deg by default)
int updateInterval = 0; // how often the servo is changing its position
long lastUpdate = 0;

long ts = 0; // checks if you can do another detection after 800 ms passed

// LCD Variables
LiquidCrystal_I2C lcd(0x27, 16, 2); // I2C address 0x27, 16 column and 2 rows
int lcd_state = 0; // lcd_state is switched based on detection
long lcd_ts = 0; // time to display the text for when an object is encountered

// Ultrasonic sensor variables
const int echoPin = 2;
const int trigPin = 3;
long duration;
int distance;
int alert_state = 0;
long alert_ts = 0;

// Buzzer variables
const int buzzer = 6; //buzzer to arduino pin 6

// LED variables
const int cyan_light_pin= 11;
const int magenta_light_pin = 10;
const int yellow_light_pin = 9;

int cyan_state = 255;
int magenta_state = 0;
int yellow_state = 255;

void greenState() {
  cyan_state = 255;
  magenta_state = 0;
  yellow_state = 255;
  CMY_color(cyan_state, magenta_state, yellow_state);
}

void redState() 
{
  cyan_state = 0;
  magenta_state = 255;
  yellow_state = 255;
  CMY_color(cyan_state, magenta_state, yellow_state);
}

void CMY_color(int cyan_light_value, int magenta_light_value, int yellow_light_value)
{
  analogWrite(cyan_light_pin, cyan_light_value);
  analogWrite(magenta_light_pin, magenta_light_value);
  analogWrite(yellow_light_pin, yellow_light_value);
}

void distanceCheck() {
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
  duration = pulseIn(echoPin, HIGH);
  distance = duration * 0.034 / 2;

  // Check if we can do another detection
  if ((millis() - ts >= 800) && (distance <= 10 && distance >= 0)) {
    ts  = millis();

    // Change the state of the buzzer and led
    if (alert_state == 0) {
      alert_state = 1;
      alert_ts = millis();
      tone(buzzer, 500);
      redState();
    }
     // Display meaningful words
    if (lcd_state == 0) {
      lcd_state = 1;
      lcd_ts = millis();
      lcd.clear();
      lcd.print("OBJECT AHEAD!");
    }
  }

  // after 80 ms reset the buzzer and led to default state
  if (alert_state == 1 && (millis() - alert_ts >= 80)) {
    alert_state = 0;
    greenState();
    noTone(buzzer); 
  }
  // after 1 second reset the lcd to default text
  if (lcd_state == 1 && (millis() - lcd_ts >= 1000)) {
    lcd_state = 0;
    lcd.clear();
    lcd.print("NO OBJECTS!");  
  }

  // send information to GUI app
  Serial.print(" | Distance: ");
  Serial.print(distance);
  Serial.println();
}

void setup()
{
  // Serial interface setup
  Serial.begin(9600);
  
  // LED Setup
  pinMode(cyan_light_pin, OUTPUT);
  pinMode(magenta_light_pin, OUTPUT);
  pinMode(yellow_light_pin, OUTPUT);

  greenState();

  // Servo Setup
  myservo.attach(servo_pin);
  myservo.write(servoAngle);
  updateInterval = 15;
  increment = 1;

  // LCD Setup
  lcd.init();
  lcd.backlight();
  lcd.setCursor(0, 0);
  lcd.print("NO OBJECTS!");

  // Buzzer setup
  pinMode(buzzer, OUTPUT);

  // Ultrasonic setup
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
}

void update_servo()
{
  // Check if we can update the servo pos
  if((millis() - lastUpdate) > updateInterval){
      lastUpdate = millis();
      servoAngle += increment; // go to next angle
      myservo.write(servoAngle);
 
      Serial.print("Degree: ");
      Serial.print(servoAngle);

      if ((servoAngle >= 180) || (servoAngle <= 0)) // end of sweep
      {
        // reverse direction
        increment = -increment;
      }
   }
}

void loop()
{
  update_servo();
  distanceCheck();
}
