import processing.serial.*;
Serial port;

int window_length = 1000;
int window_height = 1000 / 2;
int sweeper_length = 90 * window_length / (100 * 2);
int angle = 0;
int last_angle = -1;
int last_distance = -1;
ArrayList<Point> objects;
int maxDistance = 40;

class Point {
  int angle;
  int distance;
  int time = 100;
  
  public Point(int angle, int distance) {
    this.angle = angle;
    this.distance = distance;
  }
}

void drawRadar() {
  pushMatrix();
  translate(window_length / 2, window_height);
  strokeWeight(3);
  stroke(0, 255, 0);

  for (int i = 3; i <= 9 ; i += 2)
    arc(0, 0, i * window_length / 10, i * 2 * window_height / 10, PI, TWO_PI); 

  line(-sweeper_length, 0, sweeper_length, 0);
  line(-sweeper_length*cos(radians(30)), 0, sweeper_length, 0);
  for (int i = 30; i <= 150; i += 30)
    line(0, 0, -sweeper_length * cos(radians(i)), -sweeper_length * sin(radians(i))); 
  
  popMatrix();
}

void drawSweeper(int angle) {
  pushMatrix();
  translate(window_length / 2, window_height); 
  strokeWeight(3);
  stroke(255, 0, 0);
  line(0, 0, sweeper_length * cos(radians(angle)), -sweeper_length * sin(radians(angle)));
  popMatrix();
}

void drawObject(int angle, int distance) {
  pushMatrix();
  translate(window_length / 2, window_height);   
  strokeWeight(20);
  stroke(255, 0, 0); 
  
  if (distance <= 10)
    stroke(255, 0, 0);
  else 
    stroke(0, 0, 255); 
 
  point((distance * sweeper_length / maxDistance) * cos(radians(angle)),
    -(distance * sweeper_length / maxDistance) * sin(radians(angle)));
  popMatrix();
}

void drawObjects(ArrayList<Point> objects) {
  for (Point p : objects) {
    drawObject(p.angle, p.distance);
    p.time--;
  }

  for (int i = 0; i < objects.size(); ++i)
     if(objects.get(i).time <= 0)
       objects.remove(i);
}

void drawText(int angle, int distance) {
  String angleStr = "";
  String distStr = "";
  if (distance <= maxDistance && distance >= 0) {
    angleStr = angle + "°";
    distStr = distance + "cm";
  }
  
  pushMatrix();
  fill(0, 5000);
  translate(window_length * 2 / 10, window_height * 11 / 10);
  textSize(32);
  fill(0, 255, 0);
  text("Distance " + distStr, 0, 0);
  popMatrix();
  pushMatrix();
  translate(window_length * 6 / 10, window_height * 11 / 10);
  text("Angle " + angleStr, 0, 0);
  popMatrix();
  pushMatrix();
  translate(window_length * 0.35, window_height * 1.17);
  popMatrix();
}

void settings() {
  smooth();
  size(window_length, window_height * 12/10);
  objects = new ArrayList<Point>();
  port = new Serial(this, Serial.list()[0], 9600);
  
}

void draw() {
  fill(255,  50);
  rect(-10, -10, window_length * 2, window_height * 3);
  drawRadar();
  if (last_angle >= 0)
    drawText(last_angle, last_distance);
 
  if(port.available() > 0) {
    String info = port.readStringUntil('\n');
    if (info != null && info.length() >= 20) {
      String[] data = split(info, '|'); 
      String degreeStr = data[0].replaceAll("\\D+","");
      drawSweeper(Integer.valueOf(degreeStr));
      
      String distanceStr = data[1].replaceAll("\\D+","");
      if (Integer.valueOf(distanceStr) <= maxDistance) {
        last_angle = Integer.valueOf(degreeStr);
        last_distance = Integer.valueOf(distanceStr);

        objects.add(new Point(Integer.valueOf(degreeStr), Integer.valueOf(distanceStr)));
      }
    }
  }
  drawObjects(objects);
}
